#define MENU_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>

#include "interface.h"
#include "fileview.h"
#include "error_handler.h"
#include "command.h"
#include "select.h"
#include "config.h"
#include "sort.h"
#include "shell.h"
#include "find.h"
#include "dirlist.h"
#include "fof.h"
#include "dirtree.h"
#include "windows.h"

extern TViewNode *ActualFileViewList;
extern TDirNode *ActualDir;

#define NO_SELECTION_MESSAGE MessageDialog("Cannot execute command","No file selected")

void CopyButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);

  if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else FileCopyRoutine();
}

void MoveButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
  if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else FileMoveRoutine();
}

void DeleteButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
  if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else FileDeleteRoutine();
}
 
void RenameButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
 else FileRenameRoutine();
}

void AttributesButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  FileAttributesRoutine();

 AskRefreshFileList=1;
 
}

void InfoButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  FileInfoRoutine();
}
 
void CreateButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  FileCreateRoutine();
}

void ExecButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  /*  if(SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else */ FileExecRoutine();
}

void ViewButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
  if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else FileViewCommandRoutine();
}

void EditButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
  if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else FileEditCommandRoutine();
}

void TarButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveTarRoutine();
}

void UnTarButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveUnTarRoutine();
}

void GZipButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveGZipRoutine();
}

void GUnZipButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
  if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else ArchiveGUnZipRoutine();
}

void EncodeButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveEncodeRoutine();
}

void DecodeButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveDecodeRoutine();
}

void SplitButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveSplitRoutine();
}

void UnSplitButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Bool warning;

  GetVarBool("NoSelectionWarning", &warning);
 if(warning && SelectFileViewList==NULL) NO_SELECTION_MESSAGE;
  else  ArchiveUnSplitRoutine();
}

void NewFofButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  GoToDirNewWindow("");
}

void ShellWindowButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  ShellRoutine(NULL, 0);
}

void FindWindowButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  FindRoutine(NULL);
}

void FindCurWindowButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  FindRoutine(ActualDir->dir.pathname);
}

void QuitButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  RemoveWindowFromRootProperty(XtWindow(W_App));
  XFlush(XtDisplay(W_App));
  exit(0);
}

void SelectAllButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  SelectAllCommand();
}

void UnSelectAllButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  UnSelectAllCommand();
}

void SelectFilterButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  SelectFilterRoutine();
}

void RefreshTreeButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  RefreshTreeRoutine();
}

void RefreshFileListButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  AskRefreshFileList=1;
}

void HomeDirButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  HomeDirRoutine();
}

void GoToDirButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  GoToDirRoutine();
}

void SortByNameButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *info;

  info = (XmToggleButtonCallbackStruct *) widget_data;
  if(info->set==XmSET)     
    if(ActualSortType!=SORT_BY_NAME) {
      ActualSortType = SORT_BY_NAME;
      SortRoutine();
    }
  
}

void SortByTypeButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *info;

  info = (XmToggleButtonCallbackStruct *) widget_data;
  if(info->set==XmSET)     
    if(ActualSortType!=SORT_BY_TYPE) {
      ActualSortType = SORT_BY_TYPE;
      SortRoutine();
    }
}

void SortBySizeButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *info;

  info = (XmToggleButtonCallbackStruct *) widget_data;
  if(info->set==XmSET)     
    if(ActualSortType!=SORT_BY_SIZE) {
      ActualSortType = SORT_BY_SIZE;
      SortRoutine();
    }
}

void ViewLargeButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *info;

  info = (XmToggleButtonCallbackStruct *) widget_data;
  if(info->set==XmSET) 
    SelectViewMode(LARGE);
}

void ViewSmallButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *info;

  info = (XmToggleButtonCallbackStruct *) widget_data;
  if(info->set==XmSET) 
    SelectViewMode(SMALL);
}

void ViewDetailButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *info;

  info = (XmToggleButtonCallbackStruct *) widget_data;
  if(info->set==XmSET)
    SelectViewMode(DETAIL);
}


void InstallDefaultMenuCB(void)
{
  Widget w;

  /******** MENU FILE ********/

  /* FileCopy Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*CopyButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve CopyButton widget");
  XtAddCallback(w, XmNactivateCallback, CopyButtonCB, NULL);

  /* FileMove Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*MoveButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve MoveButton widget");
  XtAddCallback(w, XmNactivateCallback, MoveButtonCB, NULL);

  /* FileDelete Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*DeleteButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve DeleteButton widget");
  XtAddCallback(w, XmNactivateCallback, DeleteButtonCB, NULL);

  /* FileRename Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*RenameButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve RenameButton widget");
  XtAddCallback(w, XmNactivateCallback, RenameButtonCB, NULL);

  /* FileAttributes Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*AttributesButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve AttributesButton widget");
  XtAddCallback(w, XmNactivateCallback, AttributesButtonCB, NULL);

  /* FileCreate Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*CreateButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve CreateButton widget");
  XtAddCallback(w, XmNactivateCallback, CreateButtonCB, NULL);

  /* FileExec Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*ExecButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve ExecButton widget");
  XtAddCallback(w, XmNactivateCallback, ExecButtonCB, NULL);

  /* FileInfo Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*InfoButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve InfoButton widget");
  XtAddCallback(w, XmNactivateCallback, InfoButtonCB, NULL);

  /* FileView Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*ViewButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve ViewButton widget");
  XtAddCallback(w, XmNactivateCallback, ViewButtonCB, NULL);

  /* FileEdit Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*EditButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve EditButton widget");
  XtAddCallback(w, XmNactivateCallback, EditButtonCB, NULL);

  /* Quit Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*QuitButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve QuitButton widget");
  XtAddCallback(w, XmNactivateCallback, QuitButtonCB, NULL);

  /******** MENU ARCHIVE ********/

  /* ArchiveTar Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*TarButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve TarButton widget");
  XtAddCallback(w, XmNactivateCallback, TarButtonCB, NULL);

  /* ArchiveUnTar Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*UnTarButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve UnTarButton widget");
  XtAddCallback(w, XmNactivateCallback, UnTarButtonCB, NULL);

  /* ArchiveGZip Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*GZipButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve GZipButton widget");
  XtAddCallback(w, XmNactivateCallback, GZipButtonCB, NULL);

  /* ArchiveGUnZip Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*GUnZipButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve GUnZipButton widget");
  XtAddCallback(w, XmNactivateCallback, GUnZipButtonCB, NULL);

  /* ArchiveEncode Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*EncodeButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve EncodeButton widget");
  XtAddCallback(w, XmNactivateCallback, EncodeButtonCB, NULL);

  /* ArchiveDecode Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*DecodeButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve DecodeButton widget");
  XtAddCallback(w, XmNactivateCallback, DecodeButtonCB, NULL);

  /* ArchiveSplit Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*SplitButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve SplitButton widget");
  XtAddCallback(w, XmNactivateCallback, SplitButtonCB, NULL);

  /* ArchiveUnSplit Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*UnSplitButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve UnSplitButton widget");
  XtAddCallback(w, XmNactivateCallback, UnSplitButtonCB, NULL);

  /********* MENU SELECTION *********/


  /* Select ALL Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*SelectAllButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve SelectAllButton widget");
  XtAddCallback(w, XmNactivateCallback, SelectAllButtonCB, NULL);

  /* UnSelect ALL Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*UnSelectAllButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve UnSelectAllButton widget");
  XtAddCallback(w, XmNactivateCallback, UnSelectAllButtonCB, NULL);

  /* Select Filter Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*FilterButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve FilterButton widget");
  XtAddCallback(w, XmNactivateCallback, SelectFilterButtonCB, NULL);


  /******** MENU FILEMANAGER ********/

  /* New Fof Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*NewFofButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve NewFofButton widget");
  XtAddCallback(w, XmNactivateCallback, NewFofButtonCB, NULL);

  /* Shell Window Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*ShellWindowButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve ShellWindowButton widget");
  XtAddCallback(w, XmNactivateCallback, ShellWindowButtonCB, NULL);

  /* Find Window Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*FindWindowButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve FindWindowButton widget");
  XtAddCallback(w, XmNactivateCallback, FindWindowButtonCB, NULL);

  /* Find here Window Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*FindCurWindowButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve FindCurWindowButton widget");
  XtAddCallback(w, XmNactivateCallback, FindCurWindowButtonCB, NULL);

  /* Refresh Tree Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*RefreshTreeButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve RefreshTree Button widget");
  XtAddCallback(w, XmNactivateCallback, RefreshTreeButtonCB, NULL);

  /* Refresh Tree Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*RefreshFileListButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve RefreshFileList Button widget");
  XtAddCallback(w, XmNactivateCallback, RefreshFileListButtonCB, NULL);

  /* Home Dir Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*HomeDirButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve HomeDir Button widget");
  XtAddCallback(w, XmNactivateCallback, HomeDirButtonCB, NULL);

  /* Go To Dir Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*GoToDirButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve GoToDir Button widget");
  XtAddCallback(w, XmNactivateCallback, GoToDirButtonCB, NULL);

  /* Sort By Name Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*SortByNameButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve Sort By Name Button widget");
  XtAddCallback(w, XmNvalueChangedCallback, SortByNameButtonCB, NULL);

  /* Sort By Type Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*SortByTypeButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve Sort By Type Button widget");
  XtAddCallback(w, XmNvalueChangedCallback, SortByTypeButtonCB, NULL);

  /* Sort By Size Button */

  w = NULL;
  w = XtNameToWidget(W_App, "*SortBySizeButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve Sort By Size Button widget");
  XtAddCallback(w, XmNvalueChangedCallback, SortBySizeButtonCB, NULL);

  /* View Large Button */
  
  w = NULL;
  w = XtNameToWidget(W_App, "*ViewLargeButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve View Large Button widget");
  XtAddCallback(w, XmNvalueChangedCallback, ViewLargeButtonCB, NULL);
  
  /* View Small Button */
  
  w = NULL;
  w = XtNameToWidget(W_App, "*ViewSmallButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve View Small Button widget");
  XtAddCallback(w, XmNvalueChangedCallback, ViewSmallButtonCB, NULL);
  
  /* View Detail Button */
  
  w = NULL;
  w = XtNameToWidget(W_App, "*ViewDetailButton");
  if(w==NULL) FatalError("menu.c","InstallDefaultMenuCB","Unable to retrieve View Detail Button widget");
  XtAddCallback(w, XmNvalueChangedCallback, ViewDetailButtonCB, NULL);
  
}


























