#define OFFIX_DND_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <OffiX/DragAndDrop.h>

#include <stdio.h>

#include "dirtree.h"
#include "fileview.h"
#include "interface.h"
#include "config.h"
#include "dialogs.h"
#include "error_handler.h"

/* include the dnd icons */

#include "cursor/dir_copy.xbm"
#include "cursor/dir_move.xbm"
#include "cursor/dir_link.xbm"
#include "cursor/dir_mask.xbm"

#include "cursor/files_copy.xbm"
#include "cursor/files_move.xbm"
#include "cursor/files_link.xbm"
#include "cursor/files_mask.xbm"

/*typedef struct {
  WidgetList w_icon;
  int nb_icons;
  int nb_icons_managed;
} TDirIconList;*/

typedef enum { COPY, MOVE, LINK } TDropType; 

typedef struct {
  char *Data;
  unsigned long Size;
  char *DestDir;
  TDropType DropType;  
  int HandlingDrop;
} TDropInfos;

TDropInfos DropInfos;

extern TViewNode *ActualFileViewList;
extern TDirNode *ActualDir;

extern void FileDropExecRoutine(void);
extern void SelectionCB(Widget, XtPointer, XtPointer);

/*extern TDirIconList DirIconList;*/

char *GetDirIconName(Widget w_search, TDirIconNode *node)
{
  char *retname;
  int i;

  if(w_search==DirIconList.w_icon[node->icon_index]) {
    retname = XtMalloc(strlen(node->dir->dir.pathname)+2);
    strcpy(retname, node->dir->dir.pathname);
    strcat(retname, "/");
  } else {
    i=0;
    retname=NULL;
    while(retname==NULL && i<node->nb_children) {
      retname=GetDirIconName(w_search, node->child_icon[i]);
      i++;
    }
  }

  return(retname);
}


TDropType GetDefaultDropType(void)
{
  char *dtname;

  GetVarString("DndDefaultCommand", &dtname);
  if(strcmp(dtname, "COPY")==0) return COPY;
  if(strcmp(dtname, "MOVE")==0) return MOVE;
  if(strcmp(dtname, "LINK")==0) return LINK;

  return COPY;
}

TDropType GetCTRLDropType(void)
{
  char *dtname;

  GetVarString("DndCTRLCommand", &dtname);
  if(strcmp(dtname, "COPY")==0) return COPY;
  if(strcmp(dtname, "MOVE")==0) return MOVE;
  if(strcmp(dtname, "LINK")==0) return LINK;

  return MOVE;
}

TDropType GetSHIFTDropType(void)
{
  char *dtname;

  GetVarString("DndSHIFTCommand", &dtname);
  if(strcmp(dtname, "COPY")==0) return COPY;
  if(strcmp(dtname, "MOVE")==0) return MOVE;
  if(strcmp(dtname, "LINK")==0) return LINK;

  return LINK;
}

void DND_FileListDragHandler(Widget w, XtPointer pnt, XEvent *event, Boolean *b)
{
  int DataType;
  char *Data;
  TViewNode *actual;
  int nb_files, i, pos;
  unsigned long nb_bytes;
  int button;
  static int inhandler=0;
  int isdir;
  int xmouse, ymouse;
  Widget w_dest;
  int i_dest;
  WidgetList selection;
  XmContainerSelectCallbackStruct *infos;
  Position x,y;
  Dimension width, height;

  if(!inhandler && !DropInfos.HandlingDrop ) {
    inhandler=1;
    /* abort if button 2 isn't used */
    button = event->xmotion.state;
    if(!(button & Button2MotionMask)) {
      inhandler=0;
      return;
    }
    
    w_dest=NULL; 

    /* if no file selected, select one */
    if(SelectFileViewList==NULL) {
      xmouse=event->xmotion.x;
      ymouse=event->xmotion.y;
      if(FileIconList.nb_icons_managed>0) {
	for(i=0;i<FileIconList.nb_icons_managed;i++) {
	  XtVaGetValues(FileIconList.w_icon[i], XmNx, &x, XmNy, &y, XmNwidth, &width, XmNheight, &height, NULL);
	  if(xmouse>=x && xmouse<=x+width && ymouse>=y && ymouse<=y+height) {
	    i_dest=i;
	    w_dest=FileIconList.w_icon[i];
	  }
	}
      }
    }

    /* if we didn't choose an icon, quit */
    if(SelectFileViewList==NULL && w_dest==NULL) {
      inhandler=0;
      return;
    }

    if(w_dest!=NULL) {
      selection=(WidgetList) XtMalloc(sizeof(Widget));
      selection[0]=w_dest;
	
      XtVaSetValues(W_ListFile, XmNselectedObjectCount, 1, XmNselectedObjects, selection, NULL);

      infos = (XmContainerSelectCallbackStruct *) XtMalloc(sizeof(XmContainerSelectCallbackStruct));
      infos->selected_item_count = 1;
      infos->selected_items = selection;
    
      SelectionCB(NULL, NULL, infos);
    
      XtFree((char *) selection);
      XtFree((char *) infos);
    }

						  
    /* counting files and bytes to allocate */
    nb_files=0;
    nb_bytes=0;
    actual=SelectFileViewList;
    while(actual!=NULL) {
      if(strcmp(actual->filenode->file.name,"..")!=0) {
	nb_files++;
	nb_bytes+=strlen(actual->filenode->parent->dir.pathname)+1;
	nb_bytes+=strlen(actual->filenode->file.name)+1;
      }
      actual=actual->next;
    }

    if(nb_files==0) return;

    /* drag a single dir ? */

    isdir=0;
    if(nb_files==1 && (SelectFileViewList->filenode->file.type==DIRECTORY ||SelectFileViewList->filenode->file.type==SLINK_DIR) )
      isdir=1;

    nb_bytes++; /* last 0 char */
  
    /* allocate buffer */
    Data = XtMalloc(nb_bytes);

    /* copy data */
    pos=0;
    actual=SelectFileViewList;
    while(actual!=NULL) {
      if(strcmp(actual->filenode->file.name,"..")!=0) {
	for(i=0;i<strlen(actual->filenode->parent->dir.pathname);i++) 
	  Data[pos++]=actual->filenode->parent->dir.pathname[i];
	Data[pos++]='/';
	for(i=0;i<strlen(actual->filenode->file.name);i++) 
	  Data[pos++]=actual->filenode->file.name[i];
	Data[pos]=(char)0;
	pos++;
      }
      actual=actual->next;
    }
  
  Data[pos]=(char)0;

  DataType=DndFiles;

  DndSetData(DataType, Data, nb_bytes);

    /* copy, link or move ? */

  if(button & ControlMask)
    DropInfos.DropType=GetCTRLDropType();
  else if(button & ShiftMask)
    DropInfos.DropType=GetSHIFTDropType();
  else
    DropInfos.DropType=GetDefaultDropType();


  switch(DropInfos.DropType) {
  case COPY :
    if(isdir) 
      DndChangeCursor(DndFiles, dir_copy_width, dir_copy_height, dir_copy_bits, dir_mask_bits, dir_copy_x_hot, dir_copy_y_hot);
    else
      DndChangeCursor(DndFiles, files_copy_width, files_copy_height, files_copy_bits, files_mask_bits, files_copy_x_hot, files_copy_y_hot);
    break;
    
  case MOVE :
    if(isdir) 
      DndChangeCursor(DndFiles, dir_move_width, dir_move_height, dir_move_bits, dir_mask_bits, dir_move_x_hot, dir_move_y_hot);
    else
      DndChangeCursor(DndFiles, files_move_width, files_move_height, files_move_bits, files_mask_bits, files_move_x_hot, files_move_y_hot);
    
    break;
    
  case LINK :
    if(isdir) 
      DndChangeCursor(DndFiles, dir_link_width, dir_link_height, dir_link_bits, dir_mask_bits, dir_link_x_hot, dir_link_y_hot);
    else
      DndChangeCursor(DndFiles, files_link_width, files_link_height, files_link_bits, files_mask_bits, files_link_x_hot, files_link_y_hot);
    break;
  }    


  XtFree(Data);
  DndHandleDragging(w,event);
  inhandler=0;
  }
}

void DND_FindListDragHandler(Widget w, XtPointer pnt, XEvent *event, Boolean *b)
{
  int DataType;
  char *Data;
  int i, pos;
  unsigned long nb_bytes;
  int button;
  static int inhandler=0;
  XmStringTable seltable;
  char **selstring;
  int nbselected;
  int j;

  if(!inhandler && !DropInfos.HandlingDrop ) {
    inhandler=1;
    /* abort if button 2 isn't used */
    button = event->xmotion.state;
    if(!(button & Button2MotionMask)) {
      inhandler=0;
      return;
    }
    
    /* if no selected file the return */
    XtVaGetValues(w, XmNselectedPositionCount, &nbselected, NULL);

    if(nbselected==0) {
      inhandler=0;
      return;
    }

    XtVaGetValues(w, XmNselectedItems, &seltable, NULL);

    selstring = (char **) XtMalloc(sizeof(char *)*nbselected);

    nb_bytes=0;
    for(i=0;i<nbselected;i++) {
      XmStringGetLtoR( seltable[i], XmFONTLIST_DEFAULT_TAG, &selstring[i]);
      nb_bytes+=strlen(selstring[i])+1;
    }


    nb_bytes++; /* last 0 char */
  
    /* allocate buffer */
    Data = XtMalloc(nb_bytes);

    /* copy data */
    pos=0;
    for(i=0;i<nbselected;i++)
      for(j=0;j<strlen(selstring[i])+1;j++)
	Data[pos++]=selstring[i][j];
	  
    Data[pos]=(char)0;
	  
    for(i=0;i<nbselected;i++)
      XtFree(selstring[i]);

    XtFree( (char *) selstring);
    
    DataType=DndFiles;

  DndSetData(DataType, Data, nb_bytes);

    /* copy, link or move ? */

  if(button & ControlMask)
    DropInfos.DropType=GetCTRLDropType();
  else if(button & ShiftMask)
    DropInfos.DropType=GetSHIFTDropType();
  else
    DropInfos.DropType=GetDefaultDropType();


  switch(DropInfos.DropType) {

  case MOVE :
    DndChangeCursor(DndFiles, files_move_width, files_move_height, files_move_bits, files_mask_bits, files_move_x_hot, files_move_y_hot);
    break;

  case LINK :
    DndChangeCursor(DndFiles, files_link_width, files_link_height, files_link_bits, files_mask_bits, files_link_x_hot, files_link_y_hot);
    break;

  case COPY :
    DndChangeCursor(DndFiles, files_copy_width, files_copy_height, files_copy_bits, files_mask_bits, files_copy_x_hot, files_copy_y_hot);
  }

  XtFree(Data);
  DndHandleDragging(w,event);
  inhandler=0;
  }
}

void DND_FileListDropHandler(Widget w, XtPointer pnt, XEvent *event, Boolean *b)
{
  unsigned long DataSize;
  unsigned char *Data;
  unsigned int button;
  int i;
  Widget w_dest;
  int i_dest=0;
  TViewNode *actual;
  int dropx, dropy;
  int DestIsSource;
  int isdir;
  Position x,y;
  Dimension width, height;
  Bool warning;
 

  if(!DropInfos.HandlingDrop) {
    
    DropInfos.HandlingDrop=1;
    
    if(DndDataType(event)!=DndFiles) {
      DropInfos.HandlingDrop=0;
      return;
    }
    button = DndDragButtons(event);
    if(!(button & Button2Mask)) {
      DropInfos.HandlingDrop=0;
      return;
    }

    /* copy, link or move ? */

    /*    if(button & ControlMask) DropInfos.DropType=MOVE;
    else  if(button & ShiftMask) DropInfos.DropType=LINK;
    else DropInfos.DropType=COPY;

    fprintf(stderr,"\ndrop command\n");*/

    /* Get Drop Datas */
    DndGetData(&Data, &DataSize);
    DropInfos.Data = XtMalloc(DataSize);
    memcpy(DropInfos.Data, Data, DataSize);

    DropInfos.Size=DataSize;

    DestIsSource=0;
    if(XtWindow(w)==DndSourceWindow(event)) {
      /*      fprintf(stderr,"\ndrag source = drop destination");*/
      DestIsSource=1;
    }
    
    /* First, get the destination widget */
    w_dest=NULL;  /* no icon widget */
    if(FileIconList.nb_icons_managed>0) {
      DndDropCoordinates(w, event, &dropx, &dropy);
      for(i=0;i<FileIconList.nb_icons_managed;i++) {
	XtVaGetValues(FileIconList.w_icon[i], XmNx, &x, XmNy, &y, XmNwidth, &width, XmNheight, &height, NULL);
	if(dropx>=x && dropx<=x+width && dropy>=y && dropy<=y+height) {
	  i_dest=i;
	  w_dest=FileIconList.w_icon[i];
	}
      }
    }

    DropInfos.DestDir=NULL;

    isdir=1;
    if(w_dest!=NULL) {
      actual=ActualFileViewList;
      while(actual!=NULL) {
	if(actual->icon_index==i_dest)
	  if(actual->filenode->file.type==DIRECTORY || actual->filenode->file.type==SLINK_DIR) {
	    /*	    fprintf(stderr,"\n the file is a directory");*/
	    DropInfos.DestDir=XtMalloc(strlen(ActualDir->dir.pathname)+1+strlen(actual->filenode->file.name)+1);
	    strcpy(DropInfos.DestDir, ActualDir->dir.pathname);
	    strcat(DropInfos.DestDir, "/");
	    strcat(DropInfos.DestDir, actual->filenode->file.name);
	  }	
	  else if (actual->filenode->file.type==EXECUTABLE_FILE || (actual->filenode->file.type==SLINK_FILE && actual->filenode->file.executable)) {
	    /*	    fprintf(stderr,"\n the file is an executable\n");*/
	    DropInfos.DestDir=XtMalloc(strlen(ActualDir->dir.pathname)+1+strlen(actual->filenode->file.name)+1);
	    strcpy(DropInfos.DestDir, ActualDir->dir.pathname);
	    strcat(DropInfos.DestDir, "/");
	    strcat(DropInfos.DestDir, actual->filenode->file.name);
	    isdir=0;
	  }
	  else  {
	    /*	    fprintf(stderr,"\n the file isn't a directory");*/
	    w_dest=NULL;
	  }
	actual=actual->next;
      }
    }

    if(DropInfos.DestDir==NULL) {
      DropInfos.DestDir=XtMalloc(strlen(ActualDir->dir.pathname)+2);
      strcpy(DropInfos.DestDir, ActualDir->dir.pathname);
      strcat(DropInfos.DestDir, "/");
    }
    

    /*    fprintf(stderr,"\n Dest Dir = %s", DropInfos.DestDir);*/
    
    GetVarBool("DndWarning", &warning);

    if(w_dest==NULL && DestIsSource) {
      if(warning) MessageDialog("Drop aborted", "Cannot drag files and drop in same directory");
      XtFree(DropInfos.DestDir);
      XtFree(DropInfos.Data);
      DropInfos.HandlingDrop=0;
      return;
    }

    if (isdir) DropRoutine();
    else FileDropExecRoutine();
  }
}

void DND_TreeDirDragHandler(Widget w, XtPointer pnt, XEvent *event, Boolean *b)
{
  int DataType;
  char *Data;
  static int inhandler=0;
  Widget w_dest;
  unsigned int dropx, dropy;
  Position x, y;
  Dimension width, height;
  unsigned int i_dest;
  char *sourcedir;
  int button;
  int i;
  unsigned int nb_bytes;

  if(!inhandler && !DropInfos.HandlingDrop) {
    inhandler=1;
    /* abort if button 2 isn't used */
    button = event->xmotion.state;
    if(!(button & Button2MotionMask)) {
      inhandler=0;
      return;
    }

    /* get selected icon */

    w_dest=NULL;  /* no icon widget */
    if(DirIconList.nb_icons_managed>0) {
      dropx=event->xmotion.x;
      dropy=event->xmotion.y;
      for(i=0;i<DirIconList.nb_icons_managed;i++) {
	XtVaGetValues(DirIconList.w_icon[i], XmNx, &x, XmNy, &y, XmNwidth, &width, XmNheight, &height, NULL);
	if(dropx>=x && dropx<=x+width && dropy>=y && dropy<=y+height) {
	  i_dest=i;
	  w_dest=DirIconList.w_icon[i];
	}
      }
    }
 
    if(w_dest==NULL) {
      inhandler=0;
      return;
    }
    
    /* allocating datas */
    sourcedir=GetDirIconName(w_dest, RootIcon);
    
    if(sourcedir==NULL) {
      inhandler=0;
      return;
    }

    nb_bytes=strlen(sourcedir)+2;
    Data=XtMalloc(nb_bytes);
    strcpy(Data,sourcedir);
    Data[nb_bytes-1]=(char)0;

    XtFree(sourcedir);
    
    DataType=DndFiles;
 
    DndSetData(DataType, Data, nb_bytes);
    XtFree(Data);

    /* copy, link or move ? */
    
    if(button & ControlMask)
      DropInfos.DropType=GetCTRLDropType();
    else if(button & ShiftMask)
      DropInfos.DropType=GetSHIFTDropType();
    else
      DropInfos.DropType=GetDefaultDropType();


    switch(DropInfos.DropType) {

    case MOVE :
      DndChangeCursor(DndFiles, dir_move_width, dir_move_height, dir_move_bits, dir_mask_bits, dir_move_x_hot, dir_move_y_hot);
      break;

    case LINK :
      DndChangeCursor(DndFiles, dir_link_width, dir_link_height, dir_link_bits, dir_mask_bits, dir_link_x_hot, dir_link_y_hot);

    case COPY :
      DndChangeCursor(DndFiles, dir_copy_width, dir_copy_height, dir_copy_bits, dir_mask_bits, dir_copy_x_hot, dir_copy_y_hot);
    }
    
    DndHandleDragging(w,event);
    inhandler=0;
  }
} 
  
void DND_TreeDirDropHandler(Widget w, XtPointer pnt, XEvent *event, Boolean *b)
{
  unsigned long DataSize;
  unsigned char *Data;
  int button;
  int i;
  Widget w_dest;
  int i_dest;
  char *retname;
  int dropx, dropy;
  Position x,y;
  Dimension width, height;
  Bool warning;


  if(!DropInfos.HandlingDrop) {

    DropInfos.HandlingDrop=1;

    if(DndDataType(event)!=DndFiles) {
      DropInfos.HandlingDrop=0;
      return;
    }
    button = DndDragButtons(event);
    if(!(button & Button2Mask)) {
      DropInfos.HandlingDrop=0;
      return;
    }

    /* copy, link or move ? */

    /*    if(button & ControlMask) DropInfos.DropType=MOVE;
    else  if(button & ShiftMask) DropInfos.DropType=LINK;
    else DropInfos.DropType=COPY;


    fprintf(stderr,"\ndrop command in tree list\n");*/

    /* Get Drop Datas */

    DndGetData(&Data, &DataSize);
    DropInfos.Data = XtMalloc(DataSize);
    memcpy(DropInfos.Data, Data, DataSize);

    DropInfos.Size=DataSize;

    w_dest=NULL;  /* no icon widget */
    if(DirIconList.nb_icons_managed>0) {
      DndDropCoordinates(w, event, &dropx, &dropy);
      for(i=0;i<DirIconList.nb_icons_managed;i++) {
	XtVaGetValues(DirIconList.w_icon[i], XmNx, &x, XmNy, &y, XmNwidth, &width, XmNheight, &height, NULL);
	if(dropx>=x && dropx<=x+width && dropy>=y && dropy<=y+height) {
	  i_dest=i;
	  w_dest=DirIconList.w_icon[i];
	}
      }
    }
      
    GetVarBool("DndWarning", &warning);

    DropInfos.DestDir=NULL;
     
    if(w_dest==NULL) {
      if(warning) MessageDialog("Drop aborted", "Please drop on a directory");
      XtFree(DropInfos.Data);
      DropInfos.HandlingDrop=0;
      return;
    }

    retname = GetDirIconName(w_dest, RootIcon);
 
    if(retname==NULL) {
      if(warning) MessageDialog("Drop aborted", "Cannot find dir icon (send a bug report)");
      XtFree(DropInfos.Data);
      DropInfos.HandlingDrop=0;
      return;
    }

    DropInfos.DestDir=retname;


    DropRoutine();
  }
}


void DND_FindListInitialize(Widget w_list)
{
  DndRegisterDragWidget(w_list, DND_FindListDragHandler, NULL);
} 


void DND_FileListInitialize(Widget w_list)
{
  DndRegisterDragWidget(w_list, DND_FileListDragHandler, NULL);
  DndRegisterDropWidget(w_list, DND_FileListDropHandler, NULL);
} 

void DND_TreeDirInitialize(Widget w_list)
{
  DndRegisterDragWidget(w_list, DND_TreeDirDragHandler, NULL);
  DndRegisterDropWidget(w_list, DND_TreeDirDropHandler, NULL);
}

void DND_Initialize(Widget top_level)
{
  DndInitialize(top_level);
  DND_FileListInitialize(W_ListFile);
  DND_TreeDirInitialize(W_TreeContainer);
  DropInfos.HandlingDrop=0;
}



